<?php

namespace Drupal\noahs_page_builder\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for access checking.
 */
class CheckEntityAccessController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a CheckEntityAccessController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * Custom access check for entities.
   *
   * @param string $entity_type
   *   The entity type (e.g., 'node', 'taxonomy_term').
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  public function checkAccess($view_id, EntityInterface $taxonomy_term) {
    $config = $this->config('noahs_page_builder.settings');
    $use_in_ctype = $config->get('use_in_ctype');

    // Ensure the entity type is 'node' and the configuration is set.
    if ($view_id === 'node' && !empty($use_in_ctype)) {
      // Check if the entity's bundle matches the configured value.
      if (array_key_exists($entity->bundle(), $use_in_ctype) && $entity->bundle() === $use_in_ctype[$entity->bundle()]) {
        return AccessResult::allowed();
      }
    }

    // Deny access by default.
    return AccessResult::forbidden();
  }
}
